###################################################################################################
###################################################################################################
###											                                                                       	###
### Script of the CMR Model from                                                          ###
###												                                                                      ###
### Molenaar, D., & C?ri and Jorge L. Baz?n, M. (in press). Zero and one inflated item response ###
###       theory models for bounded continuous data. Journal of Educational and Behavioral 	    ###
###       Statistics.  										                                                      ###
### 											                                                                      ###
###	The file "CMRIRT main script.R" contains the the main script in which the code from the     ###
### other files is sourced. 
### In this script, data is simulated, the models are fit, the marginal likelihood, the DIC, and### 
### the WAIC are calculated, and the predicted density and information functions are plot.      ### 


#The code requires the following packages:
#-MASS
#-rstan
#-mvtnormComments follow #, sometimes a reference to the paper above is made											    ###
											                                                                      ###
###################################################################################################
###################################################################################################
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))


# user input
iterBayes=2000
trueModel=3             #1: beta; 2: simplex; 3: Samejima (referred to as S_B model in the paper)


#1. Simulated data
###################
nit=10
N=100
set.seed(12345)

beta=seq(.5,1,length=nit) #true intercept (difficulty)
alpha=rep(c(.5,.7),nit/2) #true slope (discrimination)
theta=rnorm(N,0,1)        #true person parameters
b0=rep(-2,nit)          #true zero inflation parameters
b1=rep(2,nit)           #true ones inflation parameters

tautrue=matrix(cbind(beta,alpha,b0,b1),ncol=4)
colnames(tautrue)=c("beta","alpha","b0","b1")
write.csv(tautrue,"tautrue.csv")
write.csv(theta,"thetatrue.csv")

#set second canonical parameter depending on the true model
if(trueModel==3) rv=rep(2,nit)       #second canonical parameter, delta_i

#init libraries (if necessary install first using "install.packages()"
library(MASS)
library(rstan)
require(simplexreg)
library(mvtnorm)

#source necessary files
source("CMRmodel fit.R")
source("CMRfunctions.R")

#compile models
source("CMRcompile model.R")

#simulate data according to the true model
if(trueModel==3) 
t=rSamejimaIRTinf(N,nit,alpha,beta,rv,theta,b0,b1)
write.csv(t,"cmrdata.csv")

#fit the models. in the case of real data, missings should be coded as -999
resSa=fitSamejimaIRTinf(t,iter=iterBayes)

tauest=resSa$output
write.csv(tauest,"tauest.csv")

attach(resSa)
thetaest=matrix(cbind(theta.est,theta.sd),ncol=2)
write.csv(thetaest,"thetaest.csv")
mcmc

#calculate model fit measures
mf_samejima=mfBoundIRTinf(resSa$mcmc,t,"samejima")

#marginal likelihoods
log(mf_samejima$r1) + mf_samejima$l_star

#DICs
mf_samejima$DIC

#WAICs
mf_samejima$WAIC

#posterior means of item parameters

print(resSa$output) # S_b IRT model (Samejima model)

#plots of density and item information
par(mfcol=c(3,2))

it=1    # which item to plot
x=seq(0.00001,.9999,by=.01)
plot(x,dBoundIRT(x,beta=resSa$beta.est[it],alpha=resSa$alpha.est[it],si2=resSa$delta.est[it],b0=resSa$b0.est[it],b1=resSa$b1.est[it],model="samejima"),main="Density S_B IRT",xlab="x",ylab="",type="l")


th=seq(-3,3,by=.01)

plot(th,InfoBoundIRTinf(th,beta=resSa$beta.est[it],alpha=resSa$alpha.est[it],si2=resSa$delta.est[it],b0=resSa$b0.est[it],b1=resSa$b1.est[it],model="samejima"),main="Information S_B IRT",xlab=expression(theta),ylab="",type="l")


save.image(file=CMR.RData)


